# -*-cperl-*-
###################################################################
###                                                             ###
###    Author: Stefan Evert                                     ###
###   Purpose: CEQL utility functions                           ###
###   Created: Thu May 25 07:26:22 2006                         ###
###                                                             ###
###################################################################
# 
# 
# 


package CEQL;

@errors = ();

sub reset_errors {
  @errors = ();
}

sub error {
  push @errors, @_;
}

sub get_errors {
  return @errors;
}

sub escape {
  my $s = shift;		# $s is a literal string => all regexp metacharacters have to be escaped
  $s =~ s/&(?![A-Za-z0-9]+;)/&amp;/g; # HTML escapes: & -> &amp; (unless it's part of an entity in the CEQL input)
  $s =~ s/</&lt;/g;
  $s =~ s/>/&gt;/g;
  # escape regular expression metacharacters used by CQP (i.e. the POSIX regexp functions)
  $s =~ s/([.*?\|\(\)\[\]\{\}\^\$])/\\$1/g; 
  $s =~ s/"/&(quot|bquo|equo);/g; # match all quote types in the BNC
  return($s)
}

sub contains_uc_latin1 {	# test whether argument contains any uppercase Latin1 characters
  return (grep {/[A-Z\xC0-\xD6\xD8-\xDE]/} @_) ? 1 : 0;
}

%lemma_categories = (
		     qw/A ADJ  N SUBST  V VERB  U UNC/,
		     map {$_ => $_} qw/ADJ ADV ART CONJ INTERJ PREP PRON SUBST VERB UNC/,
		    );
sub lookup_lemma_category {
  my $value = uc(shift);
  my $cat = $lemma_categories{$value}; # returns undef if selected category doesn't exist
  error("Invalid lemma category '$value'")
    unless defined $cat;
  return $cat;
}

%pos_codes = (			# part-of-speech "lemmas"
	      qw/A    (AJ.|CRD|DT0|ORD)   /,
	      qw/N    (N.*|ZZ0)       /,
	      qw/V    V.*           /,
	      qw/Prep (PR.*|TO0)      /,
	      qw/Ving V.G.*         /,
	      qw/Det  [AD]T.*       /,
	      qw/Adv  (AV.|XX0)       /,
	      qw/Pro  (DPS|DT[SQ]|EX0|PN.)/,
	      qw/Art  AT0 /,
	      qw/$    PU.*   \\$   PU.$ /, # current grammar turns $ into \$ (regex escapes)
	      );
# interjections and conjunctions are missing

sub lookup_pos_code {
  my $pos = ucfirst(lc(shift));
  my $regex = $pos_codes{$pos}; 
  error("Invalid part-of-speech lemma '$pos'")
    unless defined $regex;
  return $regex;
}

1;
